==> tasker-akka-2016.2.7-sources.jar: com/xebialabs/deployit/engine/tasker/PhaseExecutingActor.scala <==
package com.xebialabs.deployit.engine.tasker

import akka.actor.{ActorRef, Props}
import akka.event.LoggingReceive
import com.xebialabs.deployit.engine.api.execution.BlockExecutionState
import com.xebialabs.deployit.engine.api.execution.BlockExecutionState._
import com.xebialabs.deployit.engine.tasker.BlockExecutingActor.{BlockDone, BlockStateChanged}
import com.xebialabs.deployit.engine.tasker.messages._

//Missing Schedule
class PhaseExecutingActor(val task: Task, phase: Phase, ctx: TaskExecutionContext) extends BaseExecutionActor with ModifyStepsSupport with BecomeWithMdc {
  import context._

  private[this] val taskId = task.getId

  private[this] val block = phase.block

  //Phase are always execution on xld master side
  val blockRouter = SatelliteBlockRouter.Master

  private def logStateChange(newState: String) = debug(s">> {STATE CHANGE}: PhaseExecutingActor => [$newState]. Task: $taskId, block: $block")

  override def receive: Receive = {
    logStateChange("receive")
    LoggingReceive.withLabel("receive")(
      ReceiveWithMdc(task)(start orElse forwardStepModification(taskId, createOrLookUpBlockActor))
    )
  }

  private[this] def start: Receive = {
    logStateChange("start")
    LoggingReceive.withLabel("start") {
      case Start(`taskId`) if phase.state.isReadyForExecution =>
        info(s"[${phase.id}] : Processing Phase[${phase.id}] of task [$taskId]")
        updateStateAndNotify(EXECUTING)

        val blockActor = createOrLookUpBlockActor()

        blockActor ! Start(taskId)
        becomeWithMdc(waitingForBlockCompletion(blockActor) orElse forwardStepModification(taskId, createOrLookUpBlockActor))
    }
  }

  private[this] def createOrLookUpBlockActor() = context.child(block.id.toBlockId) match {
    case Some(ref) => ref
    case None => block match {
      case sb: StepBlock => createChild(blockRouter.route(task, sb, ctx), sb.id.toBlockId)
      case cb: CompositeBlock => createChild(blockRouter.route(task, cb, ctx), cb.id.toBlockId)
    }
  }

  private[this] def waitingForBlockCompletion(blockActor: ActorRef): Receive = {
    logStateChange("waitingForBlockCompletion")
    LoggingReceive.withLabel("waitingForBlockCompletion") {
      case BlockDone(`taskId`, `block`) =>
        debug(s"[${block.id}] : Received BlockDone message for [${block.id}]")
        updateStateAndNotify(block.state)
        notifyAndDie()

      case BlockStateChanged(`taskId`, changedBlock: Block, oldState, state) =>
        debug(s"[${block.id}] : Received BlockStateChange($oldState->$state) message for [${changedBlock.id}]")
        updateStateAndNotify(changedBlock.state)

      case stop@Stop(`taskId`) =>
        debug(s"[${block.id}] : Received Stop($taskId) message, stopping sub-block")
        blockActor ! stop

      case abort@Abort(`taskId`) =>
        debug(s"[${block.id}] : Received [$abort] message, aborting sub-block")
        blockActor ! abort
    }
  }

  private[this] def notifyAndDie() {
    if (block.state == DONE) {
      doNotAcceptMessages(s"[${block.id}] : All is [EXECUTED]")
    } else {
      become(receive)
    }

    info(s"[${block.id}] : All sub blocks of [${block.id}] completed with state ${phase.state}, notifying parent")
    parent ! BlockDone(taskId, phase)
  }

  private[this] def updateStateAndNotify(newState: BlockExecutionState) {
    val oldState: BlockExecutionState = phase.state
    if (oldState != newState) {
      phase.newState(newState)
      val msg = BlockStateChanged(task.getId, phase, oldState, newState)
      debug(s"[${phase.id}] : Sending BlockStateChanged($oldState->$newState) to ${parent.path}")
      parent ! msg
    }
  }
}

object PhaseExecutingActor {
  def props(task: Task, phase: Phase, ctx: TaskExecutionContext): Props =
    Props(classOf[PhaseExecutingActor], task, phase, ctx).withDispatcher(stateManagementDispatcher)
}

==> tasker-akka-2017.7.2-sources.jar: com/xebialabs/deployit/engine/tasker/PhaseExecutingActor.scala <==
package com.xebialabs.deployit.engine.tasker

import akka.actor.{ActorRef, Props}
import akka.event.LoggingReceive
import com.xebialabs.deployit.engine.api.execution.BlockExecutionState
import com.xebialabs.deployit.engine.api.execution.BlockExecutionState._
import com.xebialabs.deployit.engine.tasker.BlockExecutingActor.{BlockDone, BlockStateChanged}
import com.xebialabs.deployit.engine.tasker.messages._

//Missing Schedule
class PhaseExecutingActor(val task: Task, phase: Phase, ctx: TaskExecutionContext) extends BaseExecutionActor with ModifyStepsSupport with BecomeWithMdc {
  import context._

  private[this] val taskId = task.getId

  private[this] val block = phase.block

  //Phase are always execution on xld master side
  val blockRouter = SatelliteBlockRouter.Master

  private def logStateChange(newState: String) = debug(s">> {STATE CHANGE}: PhaseExecutingActor => [$newState]. Task: $taskId, block: $block")

  override def receive: Receive = {
    logStateChange("receive")
    LoggingReceive.withLabel("receive")(
      ReceiveWithMdc(task)(start orElse forwardStepModification(taskId, createOrLookUpBlockActor()))
    )
  }

  private[this] def start: Receive = {
    logStateChange("start")
    LoggingReceive.withLabel("start") {
      case Start(`taskId`) if phase.state.isReadyForExecution =>
        info(s"[${phase.id}] : Processing Phase[${phase.id}] of task [$taskId]")
        updateStateAndNotify(EXECUTING)

        val blockActor = createOrLookUpBlockActor()

        blockActor ! Start(taskId)
        becomeWithMdc(waitingForBlockCompletion(blockActor) orElse forwardStepModification(taskId, createOrLookUpBlockActor()))
    }
  }

  private[this] def createOrLookUpBlockActor() = context.child(block.id.toBlockId) match {
    case Some(ref) => ref
    case None => block match {
      case sb: StepBlock => createChild(blockRouter.route(task, sb, ctx), sb.id.toBlockId)
      case cb: CompositeBlock => createChild(blockRouter.route(task, cb, ctx), cb.id.toBlockId)
    }
  }

  private[this] def waitingForBlockCompletion(blockActor: ActorRef): Receive = {
    logStateChange("waitingForBlockCompletion")
    LoggingReceive.withLabel("waitingForBlockCompletion") {
      case BlockDone(`taskId`, `block`) =>
        debug(s"[${block.id}] : Received BlockDone message for [${block.id}]")
        updateStateAndNotify(block.state)
        notifyAndDie()

      case BlockStateChanged(`taskId`, changedBlock: Block, oldState, state) =>
        debug(s"[${block.id}] : Received BlockStateChange($oldState->$state) message for [${changedBlock.id}]")
        updateStateAndNotify(changedBlock.state)

      case stop@Stop(`taskId`) =>
        debug(s"[${block.id}] : Received Stop($taskId) message, stopping sub-block")
        blockActor ! stop

      case abort@Abort(`taskId`) =>
        debug(s"[${block.id}] : Received [$abort] message, aborting sub-block")
        blockActor ! abort
    }
  }

  private[this] def notifyAndDie() {
    if (block.state == DONE) {
      doNotAcceptMessages(s"[${block.id}] : All is [EXECUTED]")
    } else {
      become(receive)
    }

    info(s"[${block.id}] : All sub blocks of [${block.id}] completed with state ${phase.state}, notifying parent")
    parent ! BlockDone(taskId, phase)
  }

  private[this] def updateStateAndNotify(newState: BlockExecutionState) {
    val oldState: BlockExecutionState = phase.state
    if (oldState != newState) {
      phase.newState(newState)
      val msg = BlockStateChanged(task.getId, phase, oldState, newState)
      debug(s"[${phase.id}] : Sending BlockStateChanged($oldState->$newState) to ${parent.path}")
      parent ! msg
    }
  }
}

object PhaseExecutingActor {
  def props(task: Task, phase: Phase, ctx: TaskExecutionContext): Props =
    Props(classOf[PhaseExecutingActor], task, phase, ctx).withDispatcher(stateManagementDispatcher)
}
